#
# (ɤ)ץ Bread  桼󥿥ե fltk 
#
# by Noritsugu Nakamura

require "fltk"
require "uconv"
#require "iconv" if RUBY_PLATFORM =~ /(cygwin|mingw)/

# Tk ߴ󥿥ե
module Fltk
  class Widget
    def text(txt)
      self.label(Uconv.euctou8(txt))
      parent.parent.redraw # @window.redraw
    end
  end

  class Entry
    def initialize(x, y)
      @e = Fltk::Input.new(x, y)
    end
    def value
      txt = Uconv.u8toeuc(@e.value)
    end
    def value=(txt)
      @e.value= Uconv.u8toeuc(txt)
    end
    def e
      @e
    end
  end

  class UTF8Box < Box
    def initialize(*args)
      super
      labelfont(Fltk::FREE_FONT) if RUBY_PLATFORM =~ /(cygwin|mingw)/
    end
  end
end

class Dialog
  def initialize(title)
    window = Fltk::Window.new(250, 170, title)

    vpack = Fltk::Pack.new(window.w, window.h)
    vpack.spacing(20)

    box = Fltk::Box.new(80, 80, Uconv.euctou8(COPYRIGHT))
    button = Fltk::ReturnButton.new(80, 25, "OK")
    button.callback do window.hide end

    window.set_modal
    window.show
  end
end

class Help
  def initialize(title, text)
    window = Fltk::Window.new(400, 450, Uconv.euctou8(title))

    vpack = Fltk::Pack.new(window.w, window.h)
    vpack.spacing(20)
    window.add(vpack)

    d = FLTK::TextDisplay.new(380, 400)
    b = FLTK::TextBuffer.new(100)
    b.text = Uconv.euctou8(text)
    d.buffer = b

    button = Fltk::ReturnButton.new(80, 25, "OK")
    button.callback do window.hide end

    [d, button].each { |i| vpack.add(i) }

    window.show
  end
end

module UI_Fltk
  class Variable
    def initialize
      @value = ""
    end
    def value=(v)
      @value = v
    end
    attr_reader :value
  end
  COURSE = Variable.new

  def course1(w, data); change_course("50") end
  def course2(w, data); change_course("ٹ") end
  def course3(w, data); change_course("ü첻") end
  def course4(w, data); change_course("") end
  def course5(w, data); change_course("ե٥å") end
  def outotu(w, data);  change_outotu() end

  def init
    if RUBY_PLATFORM =~ /(cygwin|mingw)/
      Fltk::set_font(Fltk::FREE_FONT,
                     "a14,a12," +
                     "-*-fixed-*-*-*--*-*-*-*-*-*-jisx0201.1976-*," +
                     "-*-fixed-*-*-*--*-*-*-*-*-*-jisx0208.1983-*," +
                     "-*-fixed-*-*-*--*-*-*-*-*-*-iso10646-1")
    end
    @window = Fltk::Window.new( 300, 310, TITLE)

    vpack = Fltk::Pack.new(@window.w,@window.h)
    vpack.spacing(10)

    @menubar = Fltk::MenuBar.new( 0, 0, 300, 30 )
    @menubar.labelfont(Fltk::FREE_FONT) if RUBY_PLATFORM =~ /(cygwin|mingw)/
    vpack.add(@menubar)

    menuitems = [ 
      [ "&File",   "",   nil,       nil, Fltk::SUBMENU ],
      [ "&Quit",  "^q",  :quit_cb,  nil, 0],
      nil,
      [ "&Course", "",   nil,     nil, Fltk::SUBMENU ],
      [ "50ѹ",           "", :course1, nil, Fltk::MENU_RADIO|Fltk::MENU_VALUE],
      [ "ٹѹ",           "", :course2, nil, Fltk::MENU_RADIO],
      [ "ü첻ѹ",         "", :course3, nil, Fltk::MENU_RADIO],
      [ "ѹ",           "", :course4, nil, Fltk::MENU_RADIO],
      [ "ե٥åȤѹ", "", :course5, nil, Fltk::MENU_RADIO|Fltk::MENU_DIVIDER],
      [ "̤ѹ",             "", :outotu, nil, 0],
      nil,
      [ "&Help",   "",   nil,     nil, Fltk::SUBMENU ],
      [ "&About",  "#a",  :about_cb,     nil, 0],
      [ "Ȥ",     "", :help_usage,     nil, 0],
      [ "ɽ", "", :help_braille,     nil, 0],
      nil
    ]
    
    # Call a helper method that expands the structure above into the @menubar
    setup_menubar( menuitems )

    #@title = Fltk::Box.new(200, 20, Uconv.euctou8(" 50  "))
    @title = Fltk::UTF8Box.new(200, 20)
    vpack.add(@title)

    @braille1 = Fltk::UTF8Box.new(200, 20, Uconv.euctou8(" "))
    vpack.add(@braille1)

    @braille2 = Fltk::UTF8Box.new(200, 20, Uconv.euctou8(" "))
    vpack.add(@braille2)

    @braille3 = Fltk::UTF8Box.new(200, 20, Uconv.euctou8(" "))
    vpack.add(@braille3)

    #@entry = Fltk::Input.new(150, 20)
    @entry = Fltk::Entry.new(150, 20)
    vpack.add(@entry.e)

    @yosidame = Fltk::UTF8Box.new(200, 20)
    vpack.add(@yosidame)

    pack = Fltk::Pack.new(300,80)
    pack.pack_type(Fltk::HORIZONTAL)

    yoshi = Fltk::UTF8Box.new(75, 20, Uconv.euctou8(""))
    @l_yosi = Fltk::Box.new(75, 20, "0")

    dame = Fltk::UTF8Box.new(75, 20, Uconv.euctou8(""))
    @l_dame = Fltk::Box.new(75, 20, "0")

    pack.group_end
    vpack.add(pack)

    @window.show

    @entry.e.callback do
      answer_check
    end
    @entry.e.when ( Fltk::WHEN_ENTER_KEY )
  end

  def main_loop
    Fltk::run
  end

  # from samples/editor.rb
  def setup_menubar( items )
    # menubar.add needs to have all levels prefixed ("File/Open"),
    # but we don't want to repeat our selves. 
    prefix_stack = Array.new
    
    items.each{ |e|
      if ! e
	# nil in the main array means that 
	# a sub-menu-group has ended
	prefix_stack.pop
	@menubar.add( "" )
      else
	# extract the data from the array entry
	text     = e[0]
	shortcut = e[1]
	callback = e[2] ? method( e[2] ).to_proc : nil
	data     = e[3]
	flags    = e[4]
	
	text2 = text
	if prefix_stack.size != 0
	  text2 = prefix_stack.join("/")+"/"+text2
	end

	@menubar.add( Uconv.euctou8(text2), 
		     shortcut, callback, data, flags )

	# if the flags have the SUBMENU bit set,
	# then the next items will be below this menu,
	# so we have to remember the name
	if (flags & Fltk::SUBMENU)==Fltk::SUBMENU
	  prefix_stack.push text
	end
      end
    }
  end # setup_menubar
  private :setup_menubar

  def sleep_s(time_s)
    time = Time.new
    if (time - time_s) > 2
      nextq
    else
      Fltk.repeat_timeout(2.0){ sleep_s(time_s) }
    end
  end

  def sleep_
    time_s = Time.new
    Fltk.add_timeout(2.0) {
      sleep_s(time_s)
    }

  end

  def quit_cb(w, data)
    @window.hide
  end

  def about_cb(w, data)
    Dialog.new('Bread')
  end

  def help_usage(w, data)
    Help.new("Ȥ", Bread::USAGE)
  end

  def help_braille(w, data)
    Help.new("ɽ", Bread::TABLE)
  end
end
